import os
import sys
import math as m

master_folder = os.getcwd()

sys.path.append(os.path.join(master_folder, "Tello-master"))
sys.path.append(os.path.join(master_folder, "marvelmind.py-master\src"))

from marvelmind import MarvelmindHedge
import tello as tl

my_increments = 3                                           #Increments decide how many meters it should go before checking new pos. Maximun increments is 5 meters

z_height = 0


def get_current_coordinates(hedge):
    hedge.dataEvent.wait()                                  #Wait for new data
    hedge.dataEvent.clear()
    #z_height = tl.get_tof()/100                            #Tello height readoff
    return hedge.position()[1], hedge.position()[2], hedge.position()[3]

def create_direction(my_pos, wanted_pos):
    x1, y1 = my_pos
    x2, y2 = wanted_pos

    dx = x2 - x1
    dy = y2 - y1

    distance = m.sqrt(dx**2 + dy**2)
    angle = m.atan2(dy, dx)
    #print(angle, 'angle radiants')
    angle_deg = m.degrees(angle)                             #Radians to degrees
    rotation = ''
    #print (angle_deg, 'original angle')

    o_angle = 90 - angle_deg                                 #rotate angle 90 deg ccw 
    #print (o_angle, 'angle-90')
    if 180 > o_angle > 0:
        rotation = 'CW'
        #print(o_angle, 'new angle')

    if o_angle > 180:
        o_angle = 360 - o_angle
        rotation = 'CCW'
        #print (o_angle, 'new angle')

    if o_angle < 0:
        o_angle = (abs(o_angle))
        rotation = 'CCW'
        
    return distance, o_angle, rotation

def find_z(my_pos, wanted_pos):
    z1 = wanted_pos
    z2 = my_pos

    z_diff = round(- (z2 - z1),2) #get two decimals
    return z_diff

def Tello_Commands_Z(elevation):
    if elevation > 0.2: #positive value, upwards
        tl.up(elevation*100)

    elif elevation < -0.2: #negative value, downwards 
        tl.down(abs(elevation)*100)

    elif 0.2 > elevation > -0.2:
        print('staying level')


def Tello_Commands(distance, rotation, orientation):
    my_rotation = rotation
    if orientation == 'CCW':                                #Rotate angle towards point
        tl.anticlockwise(rotation)
    else:
        tl.clockwise(rotation)
    
    if distance > my_increments:                            #Drone can with this if loop fly longer than 5 meters. 
        increments = distance // my_increments              #how many increments goes into the distance. 15meter in 5 increments = 3 increments
        print(increments, 'increments *', my_increments)                     #increments can be defined to how often it stops. max lenght of increment is 5 meters and shortest is 20cm.
        for i in range(int(increments)):
            tl.forward(my_increments*100)
            #x, y = get_current_coordinates(hedge)

        mudolo = round(distance % my_increments, 2)
        print(round((mudolo*100),2), 'cm mudolo')
        if mudolo > 0.2:                                    #if mudolo values are more than 20 cm
            tl.forward(mudolo*100)
        else:
            print('staying')
             
    else:
            tl.forward(distance*100)                        #distance = less than 5 meters. go distance in cm


    if orientation == 'CCW':                                #Rotate back again to '0'
        tl.clockwise(rotation)
    else:
        tl.anticlockwise(rotation)


def main():
    tl.start()
    print(tl.get_battery(), 'percentage of battery left')
    
    hedge = MarvelmindHedge(tty="COM8", adr=None, debug=False)  #Create MarvelmindHedge object

    if len(sys.argv) > 1:
        hedge.tty = sys.argv[1]

    hedge.start()  # Start receiving data from marvelmind

    tl.takeoff()

    try:
        while True:
            user_input = input("Please Enter new coordinates...")
            numbers = list(map(float, user_input.split()))
            if len(numbers) != 3:
                print('Error')
                
            else:
                new_pos = numbers
                print('Selected Coordinates; x=', new_pos[0], 'y=', new_pos[1], 'z=', new_pos[2])

            current_pos = get_current_coordinates(hedge)  
            print("Current coordinates; x=", current_pos[0], "y=", current_pos[1], 'z=', current_pos[2])

            current_pos_xy = (current_pos[0], current_pos[1])   #cord of hedge,   only x and y.
            current_pos_z = tl.get_tof()/1000                   #read height of tellodrone, only z
            print(current_pos_z, 'CYPHER 1')

            wanted_pos_xy = (new_pos[0], new_pos[1])            #cord of new pos, only x and y
            wanted_pos_z  = (new_pos[2])                        #last value of input coordinates

            z_difference = find_z(current_pos_z, wanted_pos_z)
            print(z_difference, 'CYPHER 2')

            my_Distance, my_Angle, my_Orientation = create_direction(current_pos_xy, wanted_pos_xy)  #Create path

            print('distance=', round((my_Distance), 2), 'angle=', round((my_Angle), 2), 'orientation=', my_Orientation, 'Elevation:', z_difference,'m')
        
            question_input = input('GO TO COORDINATES? y for yes')  #Doublecheck of commands, This was learned the hard way.
            if question_input == 'y':
                print('GOING TO COORDINATES')
                Tello_Commands(round((my_Distance), 2), my_Angle, my_Orientation)
                Tello_Commands_Z(z_difference)
                while True:
                    while True:
                        x, y, z = get_current_coordinates(hedge)
                        current_pos = (x, y, z)
                        better_z = tl.get_tof()/1000

                        distance_to_target   = m.sqrt((new_pos[0] - current_pos[0])**2 + (new_pos[1] - current_pos[1])**2)    #Check distance between wanted coord. and new coord.
                        distance_to_target_z = find_z(better_z, wanted_pos_z)

                        print ('direct distance to target xy:', distance_to_target, 'z', distance_to_target_z)            
                    
                        if distance_to_target <= 0.2:                                                                         #Check if distance within threshold
                            print('Im at x=', x, 'y= ', y,)
                            print('Acceptable, Reached xy coordinates.')
                            break
                        
                        else:
                            print('Im at x=', x, 'y= ', y, 'and z=', z )
                            print('Not yet reached the coordinates. Trying again...')
                            current_pos = get_current_coordinates(hedge)
                            current_pos_xy = (current_pos[0], current_pos[1])

                            my_Distance, my_Angle, my_Orientation = create_direction(current_pos_xy, wanted_pos_xy)
                            Tello_Commands(round(my_Distance, 2), my_Angle, my_Orientation)                    

                    while True:
                        if 0.2 > distance_to_target_z > -0.2:
                            print('z=' , better_z,)
                            print (distance_to_target_z,'acceptable z value, staying level') 
                            break
                            
                        else:
                            better_z = tl.get_tof()/1000
                            print('wrong z altitude, im at:', better_z,'m')
                            z_difference = find_z(current_pos_z, wanted_pos_z)
                            Tello_Commands_Z(z_difference)
                            #Continue the loop and check again
                    
                    if distance_to_target <= 0.2 and 0.2 > distance_to_target_z > -0.2:
                        print('In both xy and z, finished')
                        break

            else: 
                print('not ok, LANDING')
                tl.land()

            x, y, z = get_current_coordinates(hedge)
            print('NEW COORDINATES x=',x, 'y=', y, 'and z =', z)
            
    except KeyboardInterrupt:
        hedge.stop()  # stop receiving data

if __name__ == "__main__":
    main()
